;+
; NAME: rbsp_corotation_efield
;
; PURPOSE: Determine the co-rotation electric field for RBSP. Saves as tplot variable
;		
;
; INPUT: probe -> 'a' or 'b'
;		 date  -> ex '2012-10-13'
;
; KEYWORDS: no_spice_load -> don't load SPICE kernels
;
;
; NOTES: 1) I've tested this with the hires mag data as well as lowres 4-sec data.
;		 The lowres data work just as well. 
;		 2) uses accurate 1-min cadence spinaxis pointing direction
;
; HISTORY:
;   2013-07-08: Adapted by Aaron Breneman from program by Scott Thaller, University of Minnesota
;
;
; VERSION:
; $LastChangedBy: aaronbreneman $
; $LastChangedDate: 2013-07-12 13:01:48 -0700 (Fri, 12 Jul 2013) $
; $LastChangedRevision: 12673 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/missions/rbsp/efw/rbsp_corotation_efield.pro $
;
;-



pro rbsp_corotation_efield,probe,date,no_spice_load=no_spice_load


;Initializing stuff...
	rbsp_efw_init
	!rbsp_efw.user_agent = ''
	timespan,date

;Load SPICE if required
	if ~keyword_set(no_spice_load) then rbsp_load_spice_kernels


;Get the spinphase variable. 
	rbsp_load_state,probe=probe,/no_spice_load,datatype='spinphase'

;Load spice for position and velocity data
	rbsp_load_spice_state,probe=probe,coord='gse',/no_spice_load
	

;Get spin-axis pointing direction once/minute
	time2=time_double(date) ; first get unix time double for beginning of day

	;number of time chunks throughout day
	ntimes = 1440.  ;1440 = once/min
	time3 = time_string(time2 + 60.*indgen(ntimes))

	strput,time3,'T',10 ; convert TPLOT time string 'yyyy-mm-dd/hh:mm:ss.msec' to ISO 'yyyy-mm-ddThh:mm:ss.msec'
	cspice_str2et,time3,et2 ; convert ISO time string to SPICE ET

	cspice_pxform,'RBSP'+strupcase(probe)+'_SCIENCE','GSE',et2,pxform

	wsc=dblarr(3,ntimes)
	wsc[2,*]=1d
	wsc_GSE=dblarr(3,ntimes)

	for qq=0l,ntimes-1 do wsc_GSE[*,qq] = pxform[*,*,qq] ## wsc[*,qq]






;load emfisis L3 magnetic field data

	rbsp_load_emfisis,probe=probe,cadence='4sec',coord='gse',level='l3'
	get_data,'rbsp'+probe+'_emfisis_l3_4sec_gse_Mag',data=Bmag_l3

	if is_struct(Bmag_l3) then store_data,'Mag_gse',data=Bmag_l3 else begin
		print,'NO MAG DATA LOADED.....RETURNING'
		return
	endelse



;Find corotation electric field

	;use GEI (Geocentric Equitorial Inertial) coordinates, 
	;they have Z along earth's rotation axis
	;X gei is the along the intersection of the ecliptic and 
	;equatorial planes, and point towards the first point of Aries
	;the position of the sun at the vernal equinox, and Y=ZxX 
	;GEI coordinates do not themselves rotate with the earth

	cotrans,'rbsp'+probe+'_state_pos_gse','rbsp'+probe+'_state_pos_gei',/GSE2GEI

	get_data,'rbsp'+probe+'_state_pos_gei',data=gei_pos

	if ~is_struct(gei_pos) then begin
		print,'NO POSITION DATA LOADED.....RETURNING'
		return
	endif

	gei_pos_times = gei_pos.x
	gei_pos       = gei_pos.y

	xgei = gei_pos[*,0]
	ygei = gei_pos[*,1]
	zgei = gei_pos[*,2]

	Omega_E = (2*!Pi)/(24.0*3600.0) ;Earth's rotation angular frequency

	Vel_coro = fltarr(n_elements(xgei),3) ;set up array for velocity of corotating frame at RBSP location 

	;V=Omega X position  (rigid body rotation) 
	Vel_coro[*,0] = -ygei*Omega_E  
	Vel_coro[*,1] = xgei*Omega_E
	Vel_coro[*,2] = 0.0



;Transform magnetic field into GEI coordinates
	cotrans,'Mag_gse','Mag_gei',/GSE2GEI

	get_data,'Mag_gei',data=Bgei
	mag_times=Bgei.x
	Bgei = Bgei.y

	Vel_coro=interp(Vel_coro,gei_pos_times,mag_times,/no_extrap)

	store_data,'Vel_coro_GEI',data={x:mag_times,y:Vel_coro}

	E_coro=fltarr(n_elements(mag_times),3)
	for xx=0L,n_elements(mag_times)-1 do E_coro[xx,*]=-crossp(Vel_coro[xx,*],Bgei[xx,*])/1000.0 

	store_data,'E_coro_GEI',data={x:mag_times,y:E_coro},dlim={colors:[2,4,6],lables:['Ex','Ey','Ez']}


;Transform corotational E in GEI into GSE and MGSE

	cotrans,'E_coro_GEI','E_coro_GSE',/GEI2GSE
	get_data,'E_coro_GSE',data=Ecoro

	if ~is_struct(Ecoro) then begin
		print,'NO CO-ROTATION EFIELD DATA LOADED'
		return
	endif


	;put the pointing direction on the same timestamps as the Efield data
	wsc_GSE2 = [[interpol(wsc_GSE[0,*],time_double(time3),Ecoro.x)],$
				[interpol(wsc_GSE[1,*],time_double(time3),Ecoro.x)],$
				[interpol(wsc_GSE[2,*],time_double(time3),Ecoro.x)]]
	if is_struct(Ecoro) then rbsp_gse2mgse,'E_coro_GSE',reform(wsc_GSE2),newname='E_coro_mgse'



	get_data,'E_coro_mgse',data=Ecorot
	store_data,'rbsp'+probe+'_E_coro_mgse',data=Ecorot,dlim={colors:[2,4,6],labels:['Ex','Ey','Ez']} 


end